@@ -4,7 +4,7 @@ from typing import List
 
 from jsonrpcclient.clients.http_client import HTTPClient
 
-import logging
+from app import logging
 from common.cs_enum import AutoName
 
 DEFAULT_LEDGER_BOOK_ID = "1"
@@ -14,6 +14,7 @@ log = logging.getLogger(__name__)
 
 class OperationType(AutoName):
     TRANSFER = auto()
+    VOID = auto()
 
 
 class OperationStatus(AutoName):
@@ -51,8 +52,18 @@ class LedgerClient:
     def get_book(self, book_id: str):
         return self._execute("getBook", book_id, read_only_method=True)
 
-    def get_book_balances(self, book_id: str, asset_id: str = None, metadata_filter: dict = None):
-        return self._execute("getBalances", book_id, asset_id, metadata_filter, read_only_method=True)
+    def put_book(self, book_id: str, restrictions: dict):
+        return self._execute("putBook", book_id, restrictions)
+
+    def freeze_book(self, book_id: str):
+        return self.put_book(book_id=book_id, restrictions={"freeze": True})
+
+    def unfreeze_book(self, book_id: str):
+        return self.put_book(book_id=book_id, restrictions={"freeze": False})
+
+    def get_book_balances(self, book_id: str, asset_id: str = None, metadata_filter: dict = None, timestamp: int = None):
+        filter = {"toTime": timestamp} if timestamp else None
+        return self._execute("getBalances", book_id, asset_id, metadata_filter, filter, read_only_method=True)
 
     def get_operation(self, operation_id: str):
         return self._execute("getOperation", operation_id, read_only_method=True)
@@ -60,15 +71,18 @@ class LedgerClient:
     def get_operations(self, book_id: str, metadata_filter: dict = None):
         return self._execute("getOperations", book_id, metadata_filter, read_only_method=True)
 
-    def post_operation(self, operation_type: OperationType, entries: List[dict], memo: str = "", metadata: dict = None, sync: bool = False):
+    def post_operation(self, operation_type: OperationType, entries: List[dict], memo: str = "", metadata: dict = None):
         operation = dict()
         operation["type"] = operation_type.value
         operation["entries"] = entries
         operation["memo"] = memo
         if metadata: operation["metadata"] = metadata
-        return self._execute("postOperation", operation, sync)
+        result = self._execute("postOperation", operation)
+        if result["status"] == OperationStatus.REJECTED.value:
+            raise Exception(result["rejectionReason"])
+        return result
 
-    def post_transfer(self, from_book_id: str, to_book_id: str, asset_id: str, value: str, memo: str = "", metadata: dict = None, sync: bool = False):
+    def post_transfer(self, from_book_id: str, to_book_id: str, asset_id: str, value: str, memo: str = "", metadata: dict = None):
         transfer = dict()
         transfer["fromBookId"] = from_book_id
         transfer["toBookId"] = to_book_id
@@ -76,11 +90,14 @@ class LedgerClient:
         transfer["value"] = value
         transfer["memo"] = memo
         if metadata: transfer["metadata"] = metadata
-        return self._execute("postTransfer", transfer, sync)
-
-    def post_transfer_sync(self, from_book_id: str, to_book_id: str, asset_id: str, value: str, memo: str = "", metadata: dict = None):
-        result = self.post_transfer(from_book_id=from_book_id, to_book_id=to_book_id, asset_id=asset_id, value=value, memo=memo, metadata=metadata, sync=True)
+        result = self._execute("postTransfer", transfer)
         if result["status"] == OperationStatus.REJECTED.value:
             raise Exception(result["rejectionReason"])
         return result
 
+    def post_void(self, operation_id_list: list, memo: str, metadata: dict):
+        void = dict()
+        void["operationIds"] = operation_id_list
+        void["memo"] = memo
+        void["metadata"] = metadata or {}
+        return self._execute("postVoid", void)
